import React from 'react';
import './SearchForm.css';
import TextField from 'material-ui/TextField';
import SearchHandler from '../search-handler/SearchHandler';
import Paper from 'material-ui/Paper';
import List, { ListItem, ListItemText } from 'material-ui/List';
import Divider from 'material-ui/Divider';
import Button from 'material-ui/Button';
import { InputLabel } from 'material-ui/Input';
import { MenuItem } from 'material-ui/Menu';
import { FormControl } from 'material-ui/Form';
import Icon from 'material-ui/Icon';
import IconButton from 'material-ui/IconButton';
import Select from 'material-ui/Select';
import Chip from 'material-ui/Chip';

class SearchForm extends React.Component {

  constructor(props) {
    super(props);

    this.state = {
      selectedField: '',
      searchClauses: [],
      value: '',
      showSuggestions: false,
      suggestions: null
    };
    this.termFieldTitles = {
      'title_s': 'Title',
      'author_ss': 'Author',
      'tags_ss': 'Tag'
    };
    this.termFields = ['title_s', 'author_ss', 'tags_ss'];
    this.searchHandler = new SearchHandler({ termFields: this.termFields });
    this.handleChange = this.handleChange.bind(this);
    this.handleSubmit = this.handleSubmit.bind(this);
  }

  handleChange(event) {
    let val = event.target.value.trim();
    this.setState({value: event.target.value});

    this.searchHandler.fetchTerms(val, (terms) => {
      let termCount = this.termFields.map((t) => terms[t].length)
                                     .reduce((acc,val) => acc + val, 0);
      this.setState({ showSuggestions: termCount !== 0, suggestions: terms });
    });
  }

  deleteClause(clause) {
    const searchClauses = [...this.state.searchClauses];
    const clauseToDelete = searchClauses.indexOf(clause);
    searchClauses.splice(clauseToDelete, 1);
    this.setState({ searchClauses });
  }

  addClause() {
    if (this.state.value) {

      var searchClauses = this.state.searchClauses.slice();

      searchClauses.push({
        field: this.state.selectedField,
        term: this.state.value
      });

      this.setState({
        searchClauses: searchClauses,
        selectedField: '',
        value: ''
      });
    }
  }

  applySuggestion(field, term) {
    this.setState({
      selectedField: field,
      value: term,
      showSuggestions: false
    });
  }

  handleSubmit(event) {
    console.log("Form submitted: " + this.state.value);

    this.setState({
      showSuggestions: false
    });

    if (this.state.value.trim() !== "") {
      this.addClause();
    }

    let searchQuery = "*:*";

    if (this.state.searchClauses.length > 0) {
      searchQuery = this.state.searchClauses.map((clause) => {
        return clause.field !== "" ? `+${clause.field}:(+"${clause.term}")` : `"+${clause.term}"`;
      }).join(" ");
    }

    this.props.onQuery(searchQuery);

    event.preventDefault();
  }

  render() {
    let listSuggestions = '';
    let closeButton = '';
    if (this.state.suggestions && this.state.showSuggestions) {
      closeButton = <Button color="secondary" onClick={() => {this.setState({showSuggestions: false})}}>
        Close 
      </Button>;
      listSuggestions = this.termFields.map((termField, i) => {
        let terms = this.state.suggestions[termField];
        let termsList = terms.map((term, j) => (
          <ListItem key={j} button onClick={() => { this.applySuggestion(termField, term) }}>
            <ListItemText primary={term} className="suggested-term" />
          </ListItem>));
        return (
          <div key={i}>
            <List>
              <ListItem className="term-field-title">{this.termFieldTitles[termField]}</ListItem>
              {termsList}
            </List>
            <Divider/>
          </div>
        );
      });
    }

    let inputProps = { autoComplete:"off" };

    return (
      <div>
        <form onSubmit={this.handleSubmit} className="search-form">
          <FormControl className="selected-field">
            <InputLabel htmlFor="field-select">Field</InputLabel>
            <Select
              value={this.state.selectedField}
              inputProps={{ id: 'field-select' }}
              onChange={(e) => { this.setState({selectedField: e.target.value}) }}>
              <MenuItem value="">All Fields</MenuItem>
              {this.termFields.map((tf, idx) => <MenuItem key={idx} value={tf}>{this.termFieldTitles[tf]}</MenuItem>)}
            </Select>
          </FormControl>

          <TextField
            id="search"
            className="search-input"
            onChange={this.handleChange}
            inputProps={inputProps}
            value={this.state.value}
          />
          <IconButton color="secondary" aria-label="Add a search clause" className="add-clause"
                      onClick={this.addClause.bind(this)}>
            <Icon>add</Icon>
          </IconButton>
          <Paper elevation={4} className="suggestion-list">
            {closeButton}
            {listSuggestions}
          </Paper>
          <Button raised={true}
            color = "primary"
            className = "search-button"
            onClick = {this.handleSubmit}>
            Search
          </Button>
        </form>

        <div className="search-clauses">
        {(this.state.searchClauses.length > 0) && <label>Search clauses: </label>}
        {this.state.searchClauses.map((clause, idx) => {
          let field = clause.field ? this.termFieldTitles[clause.field] + ':' : '';
          return <Chip
            key={idx}
            label={`${field} ${clause.term}`}
            onDelete={() => this.deleteClause(clause)}
            className="clause-chip"
            />
        })}
        </div>
      </div>
    );
  }
}
export default SearchForm;

// vi: ft=javascript.jsx
